function get_argument()

TK_EvalStr('set main .rltoolbox.f');
TK_EvalStr('set val [$main.f1.e get]');
n = evstr(TK_GetVar('val'));		 // reward for a draw (between lose=0 and win=1)

TK_EvalStr('set val [$main.f2.e get]');
k = evstr(TK_GetVar('val'));	        // step-size parameter (updates the P-values)

TK_EvalStr('set val [$main.f3.e get]');
epsilon = evstr(TK_GetVar('val'));    // in percent, 0 = no random moves at all

TK_EvalStr('set val [$main.f4.e get]');
method = evstr(TK_GetVar('val'));					// games to play

TK_EvalStr('set val [$main.f5.e get]');
alpha = evstr(TK_GetVar('val'));		

TK_EvalStr('set val [$main.f6.e get]');
Beta = evstr(TK_GetVar('val'));	

TK_EvalStr('set val [$main.f7.e get]');
change = evstr(TK_GetVar('val'));		

TK_EvalStr('set val [$main.f8.e get]');
Tasks = evstr(TK_GetVar('val'));

x_message(['Simulating';
'Please be patient!']);

[NbOptimalAction]=bandit(change,n,k,epsilon,method,alpha,Beta);

xset("window",0)
xbasc(0)
x=[1:k];
plot(x,NbOptimalAction);
xtitle('representation of results','number of tries','percentage of optimal actions');
xselect();
x_message(['Simulation is done!';
           'You got the results in the graph']);

endfunction


//bandit function

function [NbOptimalAction]=bandit(change,n,k,epsilon,method,alpha,Beta)

NbOptimalAction=zeros(k,1);

for j=1:Tasks                                    // number of time you want to run the simulation

// ************* initialization ******************

Real=rand(n,1,'normal');
[d,optimal]=max(Real);

ExpectedReward=zeros(n,1);
ExponentielExpRew=ones(n,1);
Probability=zeros(n,1);
Probability(:,1)=1/n;
Preference=zeros(n,1);

Totalplayed=0;
NbPlayed=zeros(n,1);
TotalReward=0;
AverageReward=0;

// *************** simulation *********************

for i=1:k					 // root cycle, k=number of plays

   b=rand(1,1,'uniform');

   if (b<=change) then                            // if we must change the rewards (non-stationary problem) 
      Real=Real+rand(n,1,'normal');
   end;


   if method=='pursuit' then                       // pursuit of greedy action (pursuit method)
       [d,current]=max(ExpectedReward);
       Probability=(1-Beta)*Probability;
       Probability(current,1)=Probability(current,1)+Beta/(1-Beta);
   end;


// *************** choice of the lever *************

   b=rand(1,1,'uniform');

   if (b<=epsilon) then                            // if non-greedy action

      if method=='simple' then                          // choose randomly
         current=floor(rand(1,1,'uniform')*n)+1;    

      elseif method=='softmax' then                     // choose in softmax method
             N=b*sum(Probability)/epsilon;         
             l=0;
             current=0;
                while l<=N then
                   current=current+1;
                   l=l+Probability(current,1);
                end;

      end;
      else [d,current]=max(ExpectedReward);        // else greedy action 
   end;

   if (method=='comparison' | method=='pursuit') then   // choose in comparison and pursuit methods
      N=b*sum(Probability);
      l=0;
      current=0;
         while l<=N then
            current=current+1;
            l=l+Probability(current,1);
         end;
   end;


   Reward=rand(1,1,'normal')+Real(current,1);   		// generation of the reward



// *************** observations of the reward **************

   TotalReward=TotalReward+Reward;              		// correction of current parameters
   NbPlayed(current,1)=NbPlayed(current,1)+1;   
   Totalplayed=Totalplayed+1;                   
   Preference(current,1)=Preference(current,1)+Beta*(Reward-AverageReward);

   if current==optimal then
      NbOptimalAction(i,1)=NbOptimalAction(i,1)+1;
   end;


   if alpha=='average' then                      // correction of expected reward or average reward 
          AverageReward=(AverageReward*(Totalplayed-1)+Reward)/Totalplayed;
          ExpectedReward(current,1)=(ExpectedReward(current,1)*(NbPlayed(current,1)-1)+Reward)/NbPlayed(current,1);
   else   AverageReward=AverageReward+alpha*(Reward-AverageReward);
          ExpectedReward(current,1)=ExpectedReward(current,1)+alpha*(Reward-ExpectedReward(current,1));
   end;


     if method=='softmax' then
        ExponentielExpRew(current,1)=exp(ExpectedReward(current,1));
         for l=1:n
             Probability(l,1)=ExponentielExpRew(l,1)/(sum(ExponentielExpRew)-ExponentielExpRew(l,1));
         end;

      end;
      
      if (method=='comparison' | method=='pursuit')  then
          for l=1:n
              Probability(l,1)=exp(Preference(l,1))/(sum(exp(Preference))-exp(Preference(l,1))+%eps);
          end;
      end;

end;                               // end of root cycle

// ************* end of simulation ***********

end;                               // end of all the simulations

NbOptimalAction=NbOptimalAction/20;

endfunction

