//*************************************//
//*   Reinforcement Learning Toolbox  *//
//*   MonteCarlo                      *//
//*   T. AL ANI                       *//
//*   ESIEE-ASI Paris - March 2004   *//
//*************************************//

//************************************************//
//*               Mc_Action_Select               *//
//************************************************//
function na=Mc_Action_Select(Pi, cs, Actionlist)

  nal  = 0
  csl  = Mc_Linear(cs, _s)
  val  = rand()
  
  for i=1:length(Actionlist(csl))
    action = Actionlist(csl)(i)
    if val > 0 then
      nal = action
    end
    val = val - Pi(csl,action)
  end
  
  na = Mc_Matrix_Action(nal)
  
endfunction
//************************************************//




//************************************************//
//*               Mc_Creat_Episode               *//
//************************************************//
function Episode=Mc_Creat_Episode(Pi, Actionlist)

  // Initial state 
  cs = Mdl_Init_State()

  // Dtermination de l'action  effectuer
  na = Mc_Action_Select(Pi, cs, Actionlist)

  Episode = list()
  turn = 1
  Episode(turn) = [cs na]

  while Mdl_Stop(Episode) == 0   // While episode is not terminated
    cs = Mdl_Get_State(Episode)
    na = Mc_Action_Select(Pi, cs, Actionlist)
    turn = turn + 1
    Episode(turn) = [cs na]
  end

endfunction
//************************************************//




//************************************************//
//*               Mc_First_Occurrence            *//
//************************************************//
function t=Mc_First_Occurrence(Episode, debut, turn)

  t = turn

  for i=debut:turn
    if Episode(i) == Episode(turn) then
      t = i
      break
    end
  end
  
endfunction
//************************************************//




//************************************************//
//*             Mc_Lin_State_Action               *//
//************************************************//
function [state, action]=Mc_Lin_State_Action(M)

  m = length(_s)
  N = M(1:m)
  state   = Mc_Linear(N, _s)
  
  f = length(M)
  N = M(m+1:f)
  action = Mc_Linear(N, _a)
  
endfunction
//************************************************//




//************************************************//
//*                 Mc_Linear                    *//
//************************************************//
function val=Mc_Linear(cn, _l)

  val = cn(1)
  f = length(cn)
  
  if val <> 0 then
    for i=2:f
      val = val + (cn(i)-1)*prod(_l(1:i-1))
    end
  end
  
endfunction
//************************************************//




//************************************************//
//*             Mc_Matrix_State                   *//
//************************************************//
function ns = Mc_Matrix_State(nsl)

  ns = []
  sz = length(_s)
  
  for i=1:sz
    n = 1
    x = sz-i
    p = prod(_s(1:x))
    while nsl > p
      nsl = nsl - p
      n = n + 1
    end
    ns = [n ns]
  end

endfunction
//************************************************//




//************************************************//
//*             Mc_Matrix_Action                 *//
//************************************************//
function na = Mc_Matrix_Action(nal)

  na = []
  sz = length(_a)
  
  for i=1:sz
    n = 1
    x = sz-i
    p = prod(_a(1:x))
    while nal > p
      nal = nal - p
      n = n + 1
    end
    na = [n na]
  end

endfunction
//************************************************//




//************************************************//
//*             Mc_Disp_Mat_S_A                  *//
//************************************************//
function Mstring=Mc_Disp_Mat_S_A(M, Titre)

  sM       = size(M)
  Mstring  = string( zeros([size(M)] + 1) )

  for i=2:sM(1)+1
  
    Mstring(i,1) = "State " + string(i-1)
    for j=2:sM(2)+1
      Mstring(1,j) = "Action " + string(j-1)
      Mstring(i,j) = "   " + string( M(i-1,j-1) )
    end
    
  end

  Mstring(1,1) = " " + Titre
  
  disp(Mstring)
    
endfunction
//************************************************//

