//*************************************//
//*   Reinforcement Learning Toolbox  *//
//*   MonteCarlo                      *//
//*   T. AL ANI                       *//
//*   ESIEE-ASI Paris - March 2004   *//
//*************************************//

//************************************************//
//*            Mc_Off_Policy_tau                 *//
//************************************************//
function tau=Mc_Off_Policy_tau(Episode, Pi)
  
  fin = length(Episode)
  tau = 0
  
  for i=1:fin
    [state, action] = Mc_Lin_State_Action(Episode(i))
    
    a = 1
    while Pi(state, a) <> 1
      a = a + 1
    end
    
    if a <> action then
      tau = i
    end
  end
  
endfunction
//************************************************//

//************************************************//
//*        Mc_Off_Policy_Evaluation              *//
//************************************************//
function [N, D, Q]=Mc_Off_Policy_Evaluation(Episode, tau, PiPrime, N, D, Q)

  start = tau + 1

  for turn=start:length(Episode)

    t = Mc_First_Occurrence(Episode, start, turn)
    R = Mdl_Reward(Episode,t)

    [state, action] = Mc_Lin_State_Action(Episode(turn))

    w=1
    for k=t+1:length(Episode)
      cp = Episode(k)
      [ke, ka] = Mc_Lin_State_Action(cp)
      w = w/PiPrime(ke,ka)
    end

    N(state,action) = N(state,action) + w*R
    D(state,action) = D(state,action) + w
    
    Q(state,action) = N(state,action) / D(state,action)
    
  end

endfunction
//************************************************//

//************************************************//
//*          Mc_Off_Policy_Improve               *//
//************************************************//
function Pi=Mc_Off_Policy_Improve(Episode, Pi, Q, Actionlist)
  
  for turn=1:length(Episode)
  
    [state, action] = Mc_Lin_State_Action(Episode(turn))
    
    Qmax = max(Q(state,Actionlist(state)))
    
    for i=1:length(Actionlist(state))
    
      action = Actionlist(state)(i)
   
      if Q(state,action) == Qmax then
        Pi(state,action) = 1
        Qmax = Qmax + 1  // To avoid come back in the if statement
      else
        Pi(state,action) = 0
      end
      
    end
    
  end

endfunction
//************************************************//

