//*************************************//
//*   Reinforcement Learning Toolbox  *//
//*   MonteCarlo                      *//
//*   T. AL ANI                       *//
//*   ESIEE-ASI Paris - March 2004   *//
//*************************************//

//***************************************************************//
//*      Initialization of the Simulation environement          *//
//***************************************************************//

//***************************************************************//
//* Create a liste of possible actions for each state*//
Actionlist = list(); //                                         *//
ssl = prod(_s); //                                              *//
sal = prod(_a); //                                              *//
//                                                              *//
// Linearization of A(s,a)                                      *//
// Boolean Matrix of possible actions for each state            *//
Al = matrix(A, ssl, sal); //                                    *//
//                                                              *//
// Associate a matrix of authorized actions to each state       *//
for state=1:ssl //                                              *//
  Actionlist(state)=[]; //                                      *//
  for action=1:sal //                                           *//
    if Al(state,action)==1 then //                              *//
      Actionlist(state)=[Actionlist(state), action]; //         *//
    end //                                                      *//
  end //                                                        *//
end //                                                          *//
//***************************************************************//



//*************************************************//
//*       e-soft policy initialization            *//
epsilon = 0.15; // probability to choose an       *//
//                 action which is not a*         *//
//*************************************************//



//***************************************************************//
//*              On Policy Initializations                      *//
//                                                              *//
// Policy Evaluation :  Returns(s,a) & Q(s,a)                   *//
Returns = zeros(2,ssl,sal); //                                  *//
sR = size(Returns);//                                           *//
Q_on  = matrix(Returns(1,:,:), sR(2), sR(3));//                 *//
//                                                              *//
// Policy :  Pi(s,a)                                            *//
Pi_on = zeros(ssl,sal); //                                      *//
for state=1:ssl //                                              *//
  sA = length(Actionlist(state)); //                            *//
  p  = 1/sA; //                                                 *//
  for i=1:sA //                                                 *//
    action = Actionlist(state)(i); //                           *//
    Pi_on(state,action) = p; //                                 *//
  end //                                                        *//
end //                                                          *//
//***************************************************************//



//***************************************************************//
//*               Off Policy Initializations                    *//
// Policy Evaluation :  Q(s,a) ; N(s,a) ; D(s,a)                *//
Q_off  = zeros(ssl,sal); //                                     *//
N      = zeros(ssl,sal); //                                     *//
D      = zeros(ssl,sal); //                                     *//
//                                                              *//
// Policy :  Pi(s,a)                                            *//
Pi_off  = zeros(ssl,sal); //                                    *//
for state=1:ssl //                                              *//
  action = Actionlist(state)(1); //                             *//
  Pi_off(state,action) = 1; //                                  *//
end //                                                          *//
//                                                              *//
// Policy :  PiPrime(s,a)                                       *//
PiPrime = Pi_on; // we use here a e-soft policy                 *//
//***************************************************************//

