//*************************************//
//*   Reinforcement Learning Toolbox  *//
//*   Temporal Difference             *//
//*   Windy Grid World demonstration  *//
//*   T. AL ANI                       *//
//*   ESIEE-ASI Paris - March 2004   *//
//*************************************//


// Mdl Windy Grid World load.sce



//**********************//
//*                    *//
//**********************//
function s=TD_Mdl_first_state()

  s = First_State;

endfunction


//**********************//
//*                    *//
//**********************//
function retour=TD_Mdl_terminal_state(s)

  [x,y] = Grid_i2c(s);
  retour=Term_States(y,x);

endfunction


//**********************//
//*                    *//
//**********************//
function a=TD_Mdl_Policy(s)
a=%t;

endfunction


//**********************//
//*                    *//
//**********************//
function [s_return,r]=TD_Mdl_nextstate_reward(s,a)


[x y]=Grid_i2c(s);

[xprime, yprime]=Grid_next_bloc_action(a,x,y);

// now we have the final state, considering the wind.
// But this state can be out of the grid, and we could have encountedarkred
// obstacles to reach this state. So we verify that now!
// This implies that the reward depends on the way from (x,y) to (xprime,yprime)

//obstacles including borders
[xprime,yprime,r]=Grid_obstacles(x,y,xprime,yprime);


s_return=Grid_c2i(xprime,yprime);

//we reached a goal state
if TD_Mdl_terminal_state(s_return) ~= 0 then
  r = r + Reward_goal;
  // we do a sum because it's possible to have a collision with an obstacle
  // and to terminate in a goal state. So we believe that the real reward in this case
  // is the sum of the reward of the obstacle and the reward of the goal state.
end

endfunction


//**********************//
//*                    *//
//**********************//
function  [xprime,yprime] = Grid_next_bloc_action( a , x , y )

  select a,
  case GAUCHE then
    xprime=x-1;
    yprime=y;,
  case BAS then
    xprime=x;
    yprime=y+1;,
  case DROITE then
    xprime=x+1;
    yprime=y;,
  case HAUT then
    xprime=x;
    yprime=y-1;,
  end

  //Wind
  xprime=xprime+wind_X(y,x);
  yprime=yprime+wind_Y(y,x);

endfunction



//**********************//
//*                    *//
//**********************//
function [x,y]=Grid_i2c(s)

x=modulo((s-1),grid_width)+1;
y=floor((s-1)/grid_width)+1;

endfunction

//**********************//
//*                    *//
//**********************//
function s=Grid_c2i(x,y)

s=(x-1)+(y-1)*grid_width+1;

endfunction


//**********************//
//*                    *//
//**********************//
function [x1,y1,r]=Grid_obstacles(x,y,xprime,yprime)
  x1=x;
  y1=y;
  r=Reward_normal;

  Liste = bresenham(x,y,xprime,yprime);
  for i=1:size(Liste,1)
    t = Grid_type_obstacles(Liste(i,1),Liste(i,2));
    select t,

    case NORMAL then //normal case
      x1 = Liste(i,1);
      y1 = Liste(i,2);
      r = Reward_normal;

    case BORDER then //border
      x1 = min([xprime, grid_width]);
      x1 = max([x1, 1]);
      y1 = min([yprime, grid_height]);
      y1 = max([y1, 1]);

      r= Reward_border;

      break
      //end of border case

    case CLIFF then //cliff case
      s=TD_Mdl_first_state();
      [x1,y1]=Grid_i2c(s);
      r = Reward_cliff;
      break

    case WALL then //wall case
      x1 = Liste(i-1,1); //i-1>0 since we cannot start from a wall!
      y1 = Liste(i-1,2);
      r = Reward_wall;
      break

    case HOLE then //hole case
      if i>1 then
	x1 = Liste(i,1);
	y1 = Liste(i,2);
	r = Reward_hole;
	break
      end

    else //treated as a normal case
      x1 = Liste(i,1);
      y1 = Liste(i,2);
      r = Reward_normal;
    end
 end

endfunction

//**********************//
//*                    *//
//**********************//
function type_bord=Grid_type_border(x,y)

type_bord=-%inf;

if x<1 then
  type_bord=GAUCHE;
end

if y<1 then
  type_bord=BAS;
end

if x>grid_width then
  type_bord=DROITE;
end

if y>grid_height then
  type_bord=HAUT;
end

endfunction

//**********************//
//*                    *//
//**********************//
function t= Grid_type_obstacles(x,y)
t=NORMAL;
  if (x<1) | (y<1) | (x>grid_width) | (y>grid_height) then
    // we go out from the grid
    t = BORDER;
  else
    t=Obstacles(y,x);
  end
endfunction




//**********************//
//*                    *//
//**********************//
function points = bresenham(x,y,xprime,yprime)
  i=1;
  if xprime==x then
    // we have a vertical line
    if y<yprime then
      y_ini = y;
      y_end = yprime;
      j = 1;
      i = 1;
    else
      y_ini = yprime;
      y_end = y;
      j = -1;
      i = y_end-y_ini+1;
    end

    for y_i=y_ini:y_end
      points(i,1)= x;
      points(i,2)= y_i;
      i = i + j;
    end
  else
    if yprime==y then
      // we have an horizontal line
      if x<xprime then
	x_ini = x;
        x_end = xprime;
        j = 1;
        i = 1;
      else
	x_ini = xprime;
        x_end = x;
        j = -1;
        i = x_end-x_ini+1;
      end
      for x_i=x_ini:x_end
	points(i,1)= x_i;
	points(i,2)= y;
	i = i + j;
      end
    else
      //we have a diagonal line

      eps=0;//epsilon
      a=(yprime-y)/(xprime-x);//coeff direct

      if (  a>0  & a<=1 ) | ( a<0 & a>=-1 ) then
	//if the angle is between 0 and 45 degre or between 135 and 180 degre
	// ie we are closer to the X axis than to the Y axis
	if x<xprime then
	  x_ini = x;
          x_end = xprime;
	  y_i = y;
	  j = 1;
	  i = 1;
	  if y<yprime then
	    incr = +1;
	  else
	    incr = -1;
	  end
	else
	  x_ini = xprime;
	  x_end = x;
	  y_i = yprime;
	  j = -1;
	  i = x_end-x_ini+1;
	  if yprime<y then
	    incr = +1;
	  else
	    incr = -1;
	  end
	end

	for x_i=x_ini:x_end
	  points(i,1)= x_i;
	  points(i,2)= y_i;
	  i = i + j;
	  eps=eps+a;
	  if abs(eps)>0.5 then
	    y_i=y_i + incr;
	    eps=eps - incr;
	  end
	end


      else
	//if the angle is between 45 and 135 degre
	// ie we are closer to the Y axis than to the X axis

	if y<yprime then
	  y_ini = y;
          y_end = yprime;
	  x_i = x;
	  j = 1;
	  i = 1;
	  if x<xprime then
	    incr = +1;
	  else
	    incr = -1;
	  end
	else
	  y_ini = yprime;
	  y_end = y;
	  x_i = xprime;
	  j = -1;
	  i = y_end-y_ini+1;
	  if xprime<x then
	    incr = +1;
	  else
	    incr = -1;
	  end
	end

	a =1/a;
	for y_i=y_ini:y_end
	  points(i,1)= x_i;
	  points(i,2)= y_i;
	  i = i + j;
	  eps=eps+a;
	  if abs(eps)>0.5 then
	    x_i=x_i + incr;
	    eps=eps - incr;
	  end
	end
      end
    end
  end
endfunction
