//*************************************//
//*   Reinforcement Learning Toolbox  *//
//*   Temporal Difference             *//
//*   Tic_Tac_Toe demonstration	      *//
//*   T. AL ANI                       *//
//*   ESIEE-ASI Paris - March 2004   *//
//*************************************//


// comp_table.m finds the output of the lookup table given a beforestate
// approach adopted from Sebastian Siegel, ece539, 11/13/2001, 12/15/2001
// calls  framevalue.m, getframeprob.m, randomize.m
// input: m (beforestate), player, randomsuggest, lookuptable
//			(if randomsuggest==1, then it will randomly generate a suggest move)
// output: matrix (2 by dr*dc), last column contains suggested move [row;column]

function suggest=comp_table(m,player,randomsuggest,lookuptable)
[dr,dc]=size(m);

suggestmatrix=zeros(dr,dc);
tableoutput=[zeros(1,dr*dc);(1:dr*dc)];

// find all P-values for the open squares on the board:
for r=1:dr,
   for c=1:dc,
      if m(r,c)==0,
         mtemp=m;
         mtemp(r,c)=player;		// try to go there...
         [sumposcurrent,sumnegcurrent]=framevalue(mtemp);
         [line,frameprob]=getframeprob(lookuptable,sumposcurrent,sumnegcurrent);
         tableoutput(1,(r-1)*dc+c,1)=frameprob;
     		suggestmatrix(r,c)=frameprob;
     end
   end
end

tableoutput=(randomize(tableoutput,1));  // helps, if many frameprobs are equal
tableoutput=sortrows(tableoutput',1,'i')';   // sort the output of the ANN (integers in new order)

suggest=zeros(2,dr*dc);						  // initialize the output of this function (coordinates)
for i=1:dr*dc
   suggest(2,i)=modulo(tableoutput(2,i),dc);  	// derive the column
   if suggest(2,i)==0,
      suggest(2,i)=dc;
   end
   suggest(1,i)=(tableoutput(2,i)-suggest(2,i))/dc+1;  // derive the row
end  // now: best choice at the end of suggest (last column)

if randomsuggest==1, // make sure that another spot is suggested
   tableoutput(1,dr*dc)=-1; // give the former best choice the lowest probability
   tableoutput(1,1:dr*dc-1)=rand(1,dr*dc-1);
   tableoutput=sortrows(tableoutput',1,'i')'; // sort the output of the ANN (integers in new order)
  
   suggest=zeros(2,dr*dc);				// initialize the output of this function (coordinates)
   for i=1:dr*dc
      suggest(2,i)=modulo(tableoutput(2,i),dc);  	// derive the column
      if suggest(2,i)==0,							
         suggest(2,i)=dc;
      end
      suggest(1,i)=(tableoutput(2,i)-suggest(2,i))/dc+1;  // derive the row
   end  
 end
endfunction

