//*************************************//
//*   Reinforcement Learning Toolbox  *//
//*   Temporal Difference             *//
//*   Tic_Tac_Toe demonstration	      *//
//*   T. AL ANI                       *//
//*   ESIEE-ASI Paris - March 2004   *//
//*************************************//


// framevalue.m - returns all S(X) and S(O) for a a board 
// approach adopted from Sebastian Siegel, ece539, 11/27/2001, 12/15/2001
// input: board matrix
// output: [S(X), S(O)]

function [sumpos,sumneg]=framevalue(m);
[dr,dc]=size(m);                    // stores the size of m
if dr==dc, // quadratic board
   mturn=zeros(dr,dc); // generate turning matrix:
   for i=1:dr,
      mturn(i,dr-i+1)=1;
   end
   Sum=zeros(2,8);
   for i=1:dr,
      for j=1:dc,
         if m(i,j)==1,
            Sum(1,1)=Sum(1,1)+2^(-((i-1)*dc+j));
            Sum(1,2)=Sum(1,2)+2^(-((i-1)*dc+(dc-j+1)));
         elseif m(i,j)==-1,
            Sum(2,1)=Sum(2,1)+2^(-((i-1)*dc+j));
            Sum(2,2)=Sum(2,2)+2^(-((i-1)*dc+(dc-j+1)));
         end
      end
   end
   // 90 CCW
   m=(m*mturn)';
   for i=1:dr,
      for j=1:dc,
         if m(i,j)==1,
            Sum(1,3)=Sum(1,3)+2^(-((i-1)*dc+j));
            Sum(1,4)=Sum(1,4)+2^(-((i-1)*dc+(dc-j+1)));
         elseif m(i,j)==-1,
            Sum(2,3)=Sum(2,3)+2^(-((i-1)*dc+j));
            Sum(2,4)=Sum(2,4)+2^(-((i-1)*dc+(dc-j+1)));
         end
      end
   end

   // another 90 CCW
   m=(m*mturn)';
   for i=1:dr,
      for j=1:dc,
         if m(i,j)==1,
            Sum(1,5)=Sum(1,5)+2^(-((i-1)*dc+j));
            Sum(1,6)=Sum(1,6)+2^(-((i-1)*dc+(dc-j+1)));
         elseif m(i,j)==-1,
            Sum(2,5)=Sum(2,5)+2^(-((i-1)*dc+j));
            Sum(2,6)=Sum(2,6)+2^(-((i-1)*dc+(dc-j+1)));
         end
      end
   end
   // another 90 CCW
   m=(m*mturn)';
   for i=1:dr,
      for j=1:dc,
         if m(i,j)==1,
            Sum(1,7)=Sum(1,7)+2^(-((i-1)*dc+j));
            Sum(1,8)=Sum(1,8)+2^(-((i-1)*dc+(dc-j+1)));
         elseif m(i,j)==-1,
            Sum(2,7)=Sum(2,7)+2^(-((i-1)*dc+j));
            Sum(2,8)=Sum(2,8)+2^(-((i-1)*dc+(dc-j+1)));
         end
      end
   end
else // board is of rectangular shape and not a square
   Sum=zeros(2,4);
   for i=1:dr,
      for j=1:dc,
         if m(i,j)==1,
            Sum(1,1)=Sum(1,1)+2^(-((i-1)*dc+j));
            Sum(1,2)=Sum(1,2)+2^(-((i-1)*dc+(dc-j+1)));
            Sum(1,3)=Sum(1,3)+2^(-((dr-i)*dc+j));
            Sum(1,4)=Sum(1,4)+2^(-((dr-i)*dc+(dc-j+1)));
         elseif m(i,j)==-1,
            Sum(2,1)=Sum(2,1)+2^(-((i-1)*dc+j));
            Sum(2,2)=Sum(2,2)+2^(-((i-1)*dc+(dc-j+1)));
            Sum(2,3)=Sum(2,3)+2^(-((dr-i)*dc+j));
            Sum(2,4)=Sum(2,4)+2^(-((dr-i)*dc+(dc-j+1)));
         end
      end
   end
end   
Sum=sortrows(Sum',2,'i')'; 	// sort according to S(O) first
Sum=sortrows(Sum',1,'i')';	// sort according to S(X)
sumpos=Sum(1,1); // first column contains min S(X)...
sumneg=Sum(2,1); // ... and for multiple minimized S(X) the corresponding min of S(O)

endfunction
